@@ -16,11 +16,8 @@ from tqdm import tqdm
 from nltk.corpus import wordnet as wn
 
 from . import html_strip
-from . import payloader
+from . import database
 
-#import neuralcoref
-
-#### TODO: make these tag sets configurable!
 _NNJJ_ = {'JJ','JJR','JJS','NN','NNP','NNS','ADJ','NOUN'} #Nouns and Adjectives
 _VBRB_ = {'RB','RBR','RBS','RP','VB','VBD','VBG','VBN','VBP','VBZ','ADV','VERB'} #Verbs and Adverbs
 _PUNC_ = {'.',',','?','!',';',':','(',')','[',']','{','}','"','\''} #Punctuation
@@ -231,7 +228,7 @@ def skipchunk(sentence,docid,sentenceid,maxslop=4,minlength=2,maxlength=4):
 
         if term:
 
-            if (term.tag in _NNJJ_):
+            if (term.tag in _NNJJ_ and token.is_alpha):
                 if ispredicate:
                     predicate = chunkToLabel(stack,origs,start,docid,sentenceid)
                     predicates.append(predicate)
@@ -253,7 +250,7 @@ def skipchunk(sentence,docid,sentenceid,maxslop=4,minlength=2,maxlength=4):
                     stack=[]
                     origs=[]                
 
-            elif (term.tag in _VBRB_) and (term.dep not in _EXCL_DEPS_):
+            elif (term.tag in _VBRB_) and (term.dep not in _EXCL_DEPS_) and (token.is_alpha):
                 if isconcept:
                     concept = chunkToLabel(stack,origs,start,docid,sentenceid)
                     concepts.append(concept)
@@ -435,7 +432,10 @@ class Skipchunk():
     # --------------------------------------------------
 
     def saveDocument(self,doc):
-        filename = self.document_data + '/' + doc[self.idfield] + '.json'
+        if not self.cache_documents:
+            return False
+
+        filename = self.document_data + '/' + str(doc[self.idfield]) + '.json'
         with open(filename,"w") as file:
             file.write(json.dumps(doc))
 
@@ -455,8 +455,6 @@ class Skipchunk():
         
         enriched = []
 
-        payloadify = payloader.Payloader()
-
         for doc,context in self.nlp.pipe(tuples,as_tuples=True):
 
             rich = context
@@ -510,9 +508,6 @@ class Skipchunk():
                                 docpredicates[predicate.key] = []
                             docpredicates[predicate.key].append(predicate)
 
-                    #PAYLOAD PIPELINE STAGE
-                    rich[payloadfield].append(payloadify.enrich(sentence))
-
                     sentenceid += 1
             
             self.saveDocument(rich)
@@ -525,6 +520,21 @@ class Skipchunk():
         conceptgroups = groupConcepts(concepts,minlabels=minlabels)
         predicategroups = groupConcepts(predicates,minlabels=minlabels)
 
+        #If the concept is new, add it to the preflabel database
+        #If the concept existed before this batch, get the existing preflabel
+        db = database.Database(self.database)
+        for idx in range(len(conceptgroups)):
+            preflabel = db.upsert_concept(conceptgroups[idx])
+            if preflabel != conceptgroups[idx].preflabel:
+                conceptgroups[idx].preflabel = preflabel
+                #print('Concept label override!',preflabel)
+        for idx in range(len(predicategroups)):
+            preflabel = db.upsert_concept(predicategroups[idx])
+            if preflabel != predicategroups[idx].preflabel:
+                predicategroups[idx].preflabel = preflabel
+                #print('Predicate label override!',preflabel)
+        db.close()
+
         self.enriched = enriched
         self.concepts = concepts
         self.predicates = predicates
@@ -537,6 +547,10 @@ class Skipchunk():
 
     def load(self,path=None):
 
+        if not self.cache_pickle:
+            print("Pickle load cancelled, you must explicitly set cache_pickle=True when initializing Skipchunk")
+            return False
+
         if not path:
             path = self.pickle_data
 
@@ -560,6 +574,10 @@ class Skipchunk():
 
     def save(self,path=None):
 
+        if not self.cache_pickle:
+            print("Pickling cancelled, you must explicitly set cache_pickle=True when initializing Skipchunk")
+            return False
+
         if not path:
             path = self.pickle_data
 
@@ -595,7 +613,9 @@ class Skipchunk():
             subjectof_dependencies = _SUBJ_DEPS_,
             objectof_dependencies = _OBJ_DEPS_,
             exclude_tags = _EXCL_,
-            exclude_dependencies = _EXCL_DEPS_
+            exclude_dependencies = _EXCL_DEPS_,
+            cache_documents = False,
+            cache_pickle = False
         ):
 
         #Config:
@@ -614,7 +634,6 @@ class Skipchunk():
         #Initialize NLP pipeline
         self.spacy_model=spacy_model
         self.nlp = spacy.load(self.spacy_model)
-        #neuralcoref.add_to_pipe(self.nlp)
 
         #These don't do anything yet but they will be used later to untangle the global constants
         self.concept_tags = concept_tags
@@ -625,6 +644,20 @@ class Skipchunk():
         self.exclude_tags = exclude_tags
         self.exclude_dependencies = exclude_dependencies
 
+        #Enable at your own risk!  This will take up about 2-3x your raw content size on disk
+        self.cache_documents = cache_documents
+        if not self.cache_documents:
+            print('Enriched Documents WILL NOT be saved to disk! To enable this, call Skipchunk with cache_documents=True')
+        else:
+            print('Enriched Documents WILL be saved to disk! To disable this, call Skipchunk with cache_documents=False')
+
+        #Enable at your own risk!  High content volumes will make you slow or even crash
+        self.cache_pickle = cache_pickle
+        if not self.cache_pickle:
+            print('Skipchunk WILL NOT pickle! To enable this, call Skipchunk with cache_pickle=True')
+        else:
+            print('Skipchunk WILL pickle! To disable this, call Skipchunk with cache_pickle=False')
+
         #Stateful data that will change when Skipchunk.enrich is run:
         self.enriched = None
         self.concepts = dict()
@@ -651,32 +684,12 @@ class Skipchunk():
         if not os.path.isdir(self.pickle_data):
             os.makedirs(self.pickle_data)
 
+        self.sqlite_data = os.path.join(self.root, 'sqlite')
+        self.database = os.path.join(self.sqlite_data, 'skipchunk.db')
+        print(self.database)
+        if not os.path.isdir(self.sqlite_data):
+            os.makedirs(self.sqlite_data)
+
         self.document_data = os.path.join(self.root, 'documents')
         if not os.path.isdir(self.document_data):
             os.makedirs(self.document_data)
-
-
-        ### THE BIG PICTURE ######################################
-        """
-        CONTENT ANALYSIS PIPELINE
-         - tuplize (decide what fields to use and what contexts)
-         - html strip (strips html from fields into plain text)
-         - sentencize (splits plain text into sentences)
-         - enrich (extracts the latent language graph)
-         - entitize (flags entities)
-         - payload (assigns tokens a payload value)
-        """
-
-        """
-        QUERY ANALYSIS PIPELINE
-         - shingler (left-most longest shingles for concept identification)
-         - tagger (looks up shingles in the concept dictionary)
-         - rewriter (rewrites the query based on the tags)
-        """
-
-        """
-        COMMANDS
-         - analyze (executes the above pipeline)
-         - index (sends enriched documents to the search engine)
-         - save or load (saves to the local disk)
-        """
\ No newline at end of file
